#
#
# Author: SEPP HOCHREITER
###############################################################################


fabia <- function(X,cyc,alpha,spl,spz,p,random=NULL,scale=0.0,norm=1,center=2,lap=1.0){
	## X - data matrix
	## cyc - maximum number of cycles
        ## alpha - sparseness
        ## p - factors


        eps <- as.double(1e-3)
        eps1 <- as.double(1e-10)
        init_lapla <- 1.0
        init_psi <- 0.2

        X <- as.matrix(X)

        l=ncol(X)
        n=nrow(X)

        if (center>0) {
            if (center<2) {
                X <- X- apply(X, 1, mean)
            } else {
                if (center<3) {
                    X <- X- apply(X, 1, median)
                } else {
                    mo <- estimateMode(X)
                    X <- mo$xu
                }
            }
        }

        XX <- diag((1/l)*tcrossprod(X)+0.001*as.vector(rep(1,n)))

        if (norm>0) {
            dXX <- 1/sqrt(XX)
            X <- dXX*X
            XX <- as.vector(rep(1.0,n))
        }

        if (is.null(random)||(random == FALSE))
        {
            L <- matrix(rnorm(n*p),nrow=n,ncol=p)

        } else {
            if (random>0)
            {
                L <- matrix(random*rnorm(n*p),nrow=n,ncol=p)
            } else {
                L <- matrix(rnorm(n*p),nrow=n,ncol=p)
            }
        }


        if (scale>0)
        {

            dL <- 1/sqrt( diag(tcrossprod(L)+0.001*as.vector(rep(1,nrow(L)))))
            L <- (scale*dL)*L
        }


        lapla <-  init_lapla*matrix(1,nrow=l,ncol=p)

        Psi <- init_psi*XX



	cyc <- as.integer(cyc)
	alpha <- as.double(alpha)
	p <- as.integer(p)
	spz <- as.double(spz)
	scale <- as.double(scale)
	lap <- as.double(lap)



	res <- .Call("fabic", X,Psi,L,lapla,cyc ,alpha,eps,eps1,spl,spz,scale,lap,PACKAGE="fabia")


        Lz=res$L%*%res$E_SX_n

        return(list(LZ=Lz,L=res$L,Z=res$E_SX_n,Psi=res$Psi,lapla=res$lapla))
}

fabiap <- function(X,cyc,alpha,spl,spz,p,sL,sZ,random=NULL,scale=0.0,norm=1,center=2,lap=1.0){
	## X - data matrix
	## cyc - maximum number of cycles
        ## alpha - sparseness
        ## p - factors
        ## sL - final sparseness L
        ## sZ - final sparseness z


        eps <- as.double(1e-3)
        eps1 <- as.double(1e-10)
        init_lapla <- 1.0
        init_psi <- 0.2

        X <- as.matrix(X)

        l=ncol(X)
        n=nrow(X)

        if (center>0) {
            if (center<2) {
                X <- X- apply(X, 1, mean)
            } else {
                if (center<3) {
                    X <- X- apply(X, 1, median)
                } else {
                    mo <- estimateMode(X)
                    X <- mo$xu
                }
            }
        }


        XX <- diag((1/l)*tcrossprod(X)+0.001*as.vector(rep(1,n)))

        if (norm>0) {
            dXX <- 1/sqrt(XX)
            X <- dXX*X
            XX <- as.vector(rep(1.0,n))
        }

        if (is.null(random)||(random == FALSE))
        {
            L <- matrix(rnorm(n*p),nrow=n,ncol=p)
        }
        else {
            if (random>0)
            {
                L <- matrix(random*rnorm(n*p),nrow=n,ncol=p)
            } else {
                L <- matrix(rnorm(n*p),nrow=n,ncol=p)
            }
        }


        if (scale>0)
        {

            dL <- 1/sqrt( diag(tcrossprod(L)+0.001*as.vector(rep(1,nrow(L)))))
            L <- (scale*dL)*L
        }


        lapla <-  init_lapla*matrix(1,nrow=l,ncol=p)

        Psi <- init_psi*XX


	cyc <- as.integer(cyc)
	alpha <- as.double(alpha)
	p <- as.integer(p)
	spz <- as.double(spz)
	scale <- as.double(scale)
	lap <- as.double(lap)


	res <- .Call("fabic", X,Psi,L,lapla,cyc,alpha,eps,eps1,spl,spz,scale,lap,PACKAGE="fabia")

        rL <- res$L
        n <- nrow(rL)
        gh1 <- sqrt(1.0*n)-(sqrt(1.0*n)-1.0)*sL
        for (i in 1:p)
        {
            #le <- sum(res$L[,i]^2)
            le <- 1.0
            rL[,i] <- projfunc(s=as.vector(res$L[,i]),k1=gh1,k2=le)
        }

        rz <- res$E_SX_n
        l <- ncol(rz)
        gh2 <- sqrt(1.0*l)-(sqrt(1.0*l)-1.0)*sZ
        for (i in 1:p)
        {
            #le <- sum(res$z[,i]^2)
            le <- 1.0
            rz[i,] <- projfunc(s=as.vector(res$E_SX_n[i,]),k1=2,k2=le)
        }

        Lz=rL%*%rz

        return(list(LZ=Lz,L=rL,Z=rz,Psi=res$Psi,lapla=res$lapla))
}

fabias <- function(X,cyc,alpha,spz,p,random=NULL,norm=1,center=2,lap=1.0){
	## X - data matrix
	## cyc - maximum number of cycles
        ## alpha - sparseness low value because enforced by projfunc
        ## p - factors


       eps <- as.double(1e-3)
        eps1 <- as.double(1e-10)
        init_lapla <- 1.0
        init_psi <- 0.2

        X <- as.matrix(X)

        l=ncol(X)
        n=nrow(X)

        if (center>0) {
            if (center<2) {
                X <- X- apply(X, 1, mean)
            } else {
                if (center<3) {
                    X <- X- apply(X, 1, median)
                } else {
                    mo <- estimateMode(X)
                    X <- mo$xu
                }
            }
        }


        XX <- diag((1/l)*tcrossprod(X)+0.001*as.vector(rep(1,n)))

        if (norm>0) {
            dXX <- 1/sqrt(XX)
            X <- dXX*X
            XX <- as.vector(rep(1.0,n))
        }

        if (is.null(random)||(random == FALSE))
        {
            L <- matrix(rnorm(n*p),nrow=n,ncol=p)
        } else {
            if (random>0)
            {
                L <- matrix(random*rnorm(n*p),nrow=n,ncol=p)
            } else {
                L <- matrix(rnorm(n*p),nrow=n,ncol=p)
            }
        }


        lapla <-  init_lapla*matrix(1,nrow=l,ncol=p)

        Psi <- init_psi*XX


	cyc <- as.integer(cyc)
	alpha <- as.double(alpha)
	p <- as.integer(p)
	spz <- as.double(spz)
	lap <- as.double(lap)


	res <- .Call("fabics", X,Psi,L,lapla,cyc ,alpha,eps,spz,lap,PACKAGE="fabia")


        Lz=res$L%*%res$E_SX_n

        return(list(LZ=Lz,L=res$L,Z=res$E_SX_n,Psi=res$Psi,lapla=res$lapla))
}



fabiaVersion <- function() {
  version <- packageDescription("fabia",fields="Version")
  cat('\nFABIA Package Version ', version, '\n')
  cat('\nCopyright (c) 2009 by Sepp Hochreiter\n\n')
}


estimateMode <- function(X,maxiter=50,tol=0.001,alpha=0.1,a1=4.0,G1=FALSE) {

    l=ncol(X)
    n=nrow(X)
    u <- apply(X, 1, median)
    xu <- X - u
    XX <- sqrt(diag((1/l)*tcrossprod(xu)+0.001*as.vector(rep(1,n))))
    dXX <- 1/XX
    X <- dXX*X

    u <- apply(X, 1, median)
    gxu <- 10.0*tol
    iter <- 1
    alpha <- alpha/l

    xu <- X - u

    if (G1) {
        while (abs(gxu) > tol && iter < maxiter) {
            gxu <- apply(xu, 1, function(x) alpha*sum(tanh(a1 * x)))
            u <- u + gxu
            xu <- X - u
            iter <- iter + 1
        }
    } else {
        while (abs(gxu) > tol && iter < maxiter) {
            gxu <- apply(xu, 1, function(x) alpha*sum(x * exp(-a1*(x^2)/2)))
            u <- u + gxu
            xu <- X - u
            iter <- iter + 1
        }
    }

    u <- XX*u
    xu <- XX*xu

    return(list(u=u,xu=xu))
}

myImagePlot <- function(x,xLabels=NULL, yLabels=NULL, zlim=NULL, title=NULL){


     min <- min(x)
     max <- max(x)

      if( !is.null(zlim) ){
       min <- zlim[1]
       max <- zlim[2]
    }

    if( !is.null(yLabels) ){
       yLabels <- c(yLabels)
    } else {
       yLabels <- rownames(x)
   }

    if( !is.null(xLabels) ){
      xLabels <- c(xLabels)
    } else {
     xLabels <- colnames(x)
    }

     if( is.null(title) ){
     title <- c()
    }

# check for null values
if( is.null(xLabels) ){
   xLabels <- c(1:ncol(x))
}
if( is.null(yLabels) ){
   yLabels <- c(1:nrow(x))
}

#layout(matrix(data=c(1,2), nrow=1, ncol=2), widths=c(4,1), heights=c(1,1))
layout(matrix(data=c(1,2), nrow=1, ncol=2), widths=c(5,1), heights=c(1,1))

 # Red and green range from 0 to 1 while Blue ranges from 1 to 0
 ColorRamp <- rgb( seq(0,1,length=256),  # Red
                   seq(0,1,length=256),  # Green
                   seq(1,0,length=256))  # Blue
 ColorLevels <- seq(min, max, length=length(ColorRamp))

 # Reverse Y axis
 reverse <- nrow(x) : 1
 yLabels <- yLabels[reverse]
 x <- x[reverse,]

 # Data Map
# par(mar = c(3,5,2.5,2))
 par(mar = c(3,3,2.5,1))
 image(1:length(xLabels), 1:length(yLabels), t(x), col=ColorRamp, xlab="",
 ylab="", axes=FALSE, zlim=c(min,max))
 if( !is.null(title) ){
    title(main=title)
 }
axis(BELOW<-1, at=1:length(xLabels), labels=xLabels, cex.axis=0.7)
 axis(LEFT <-2, at=1:length(yLabels), labels=yLabels, las= HORIZONTAL<-1,
 cex.axis=0.7)

 # Color Scale
# par(mar = c(3,2.5,2.5,2))
 par(mar = c(3,2,2.5,1))
 image(1, ColorLevels,
      matrix(data=ColorLevels, ncol=length(ColorLevels),nrow=1),
      col=ColorRamp,
      xlab="",ylab="",
      xaxt="n")

 layout(1)
}




PlotBicluster <- function(x,samples,observations,xLabels=NULL, yLabels=NULL, zlim=NULL, title=NULL,x11b=TRUE){


     n <- nrow(x)


     l <- ncol(x)

     min <- min(x)
     max <- max(x)

      if( !is.null(zlim) ){
       min <- zlim[1]
       max <- zlim[2]
    }

    if( !is.null(yLabels) ){
       yLabels <- c(yLabels)
    } else {
       yLabels <- rownames(x)
   }

    if( !is.null(xLabels) ){
      xLabels <- c(xLabels)
    } else {
     xLabels <- colnames(x)
    }

     if( is.null(title) ){
     title <- c()
    }

# check for null values
if( is.null(xLabels) ){
   xLabels <- c(1:l)
}
if( is.null(yLabels) ){
   yLabels <- c(1:n)
}

if (x11b) { x11() }

layout(matrix(data=c(2,1), nrow=1, ncol=2), widths=c(5,1), heights=c(1,1))

 # Red and green range from 0 to 1 while Blue ranges from 1 to 0
 ColorRamp <- rgb( seq(0,1,length=256),  # Red
                   seq(0,1,length=256),  # Green
                   seq(1,0,length=256))  # Blue
 ColorLevels <- seq(min, max, length=length(ColorRamp))

 # Reverse Y axis
 reverse <- n : 1
 yLabels <- yLabels[reverse]
 x <- x[reverse,]

 # Color Scale
 par(mar = c(3,2,2.5,1))
 image(1, ColorLevels,
      matrix(data=ColorLevels, ncol=length(ColorLevels),nrow=1),
      col=ColorRamp,
      xlab="",ylab="",
      xaxt="n")


     obs <- match(observations,yLabels)
     samp <- match(samples,xLabels)



    ybl <- length(obs)
    pmL <- diag(n)
    for (i in 1:ybl){
        ii <- n-i+1
        pmL[ii,ii] <- 0
        pmL[ii,obs[i]] <- 1
        pmL[obs[i],obs[i]] <- 0
        pmL[obs[i],ii] <- 1
    }

    xbl <- length(samp)
    pmZ <- diag(l)
    for (i in 1:xbl){
        pmZ[i,i] <- 0
        pmZ[i,samp[i]] <- 1
        pmZ[samp[i],samp[i]] <- 0
        pmZ[samp[i],i] <- 1
    }

    x <- pmL%*%x%*%pmZ


     if (n>100)
     {
         lll <- 12

         yl <- c(1,round(n*(1:lll)/lll))

         yll <- rep("",n)

         yll[yl] <- as.character(yl)
         yLabels <- yll
     }


 # Data Map
 par(mar = c(3,3,2.5,1))
 image(1:length(xLabels), 1:length(yLabels), t(x), col=ColorRamp, xlab="",
 ylab="", axes=FALSE, zlim=c(min,max))
 if( !is.null(title) ){
    title(main=title)
 }
axis(BELOW<-1, at=1:length(xLabels), labels=xLabels, cex.axis=0.7)
 axis(LEFT <-2, at=1:length(yLabels), labels=yLabels, las= HORIZONTAL<-1,
 cex.axis=0.7)

#rect(xleft, ybottom, xright, ytop, density = NULL, angle = 45,
#     col = NA, border = NULL, lty = par("lty"), lwd = par("lwd"),
#     ...)

rect(xleft=0.6, ybottom= n-ybl, xright=xbl+1, ytop=n+0.3,lwd=3,border="red")

layout(1)




if (x11b) { x11() }

layout(matrix(data=c(2,1), nrow=1, ncol=2), widths=c(5,1), heights=c(1,1))

 # Color Scale
 par(mar = c(3,2,2.5,1))
 image(1, ColorLevels,
      matrix(data=ColorLevels, ncol=length(ColorLevels),nrow=1),
      col=ColorRamp,
      xlab="",ylab="",
      xaxt="n")


 # Data Map
 par(mar = c(3,3,2.5,1))
 image(1:xbl, 1:ybl, t(x[(n-ybl+1):n,1:xbl]), col=ColorRamp, xlab="",
 ylab="", axes=FALSE, zlim=c(min,max))
 if( !is.null(title) ){
    title(main=title)
 }
axis(BELOW<-1, at=1:xbl, labels=xLabels[1:xbl], cex.axis=0.7)
 axis(LEFT <-2, at=1:ybl, labels=yLabels[1:ybl], las= HORIZONTAL<-1,
 cex.axis=0.7)

 layout(1)
}




fabi <- function(X,cyc,alpha,spl,spz,p,norm=1,center=2){

        if(!is.matrix(X)) {
            stop("X must be a matrix")
        }


       eps <- as.double(1e-3)
        eps1 <- as.double(1e-10)
        init_lapla <- 1.0
        init_psi <- 0.2

        X <- as.matrix(X)

        l=ncol(X)
        n=nrow(X)

        if (center>0) {
            if (center<2) {
                X <- X- apply(X, 1, mean)
            } else {
                if (center<3) {
                    X <- X- apply(X, 1, median)
                } else {
                    mo <- estimateMode(X)
                    X <- mo$xu
                }
            }
        }

        XX <- diag((1/l)*tcrossprod(X)+0.001*as.vector(rep(1,n)))

        if (norm>0) {
            dXX <- 1/sqrt(XX)
            X <- dXX*X
            XX <- as.vector(rep(1.0,n))
        }



       lapla <-  init_lapla*matrix(1,nrow=l,ncol=p)

        Psi <- init_psi*XX


        eps2 <- 1e-1




        #y_v <- colMeans(X)
        #xmean <- as.vector(rep(1,l))%*%t(y_v)
        #x <- X-xmean  ## center data (0 mean)
	#XX <- tcrossprod(X,X)/n
        #dXX <- diag(diag(XX))
        # idXX <- diag(1/diag(XX))
        #XX <- XX%*%idXX
        #x <- x%*%sqrt(idXX)


        kvect <- as.vector(rep(1,p))
        nvect <- as.vector(rep(1,n))
        nk_one <- matrix(1,n,p)
        nk_zero <- matrix(0,n,p)
        kk_zero <- matrix(0,p,p)
        kk_one <- diag(p)
	epsv<-eps1*kvect
	epsn<-eps*nvect
        E_SX_n <- matrix(0,p,l)
        E_SSXX_n <- list()

        L <- (0.5*XX^(.5))*nk_one
	for (i in 1:cyc){
                ipsi <- 1/Psi
		LPsi<-diag(ipsi)%*%L
		LPsiL<-crossprod(L,LPsi)
 		sum1<- nk_zero
		sum2<- eps2*kk_one
 		for (j in 1:l){
			tmp <- solve(LPsiL+diag(lapla[j,]))
			x_j <- as.vector(X[,j])
			e_sx_n <- as.vector(tcrossprod(tmp,LPsi)%*%x_j)
			#e_sx_n[which(e_sx_n<0)] <- 0
			e_ssxx_n <-  tmp + tcrossprod(e_sx_n,e_sx_n)
			sum1 <- sum1 +  tcrossprod(x_j,e_sx_n)
			sum2 <- sum2 + e_ssxx_n
			lapla[j,] <- (epsv+diag(e_ssxx_n))^(-spz)
                        lapla[j,which( lapla[j,]<1.0)] <- 1.0
		}
#                L <- (sum1 - alpha*sign(sum1))
#                L[which(abs(sum1)<alpha)] <- 0
                sll <- solve(sum2)
                L <- sum1%*%sll
                L <- L - alpha*Psi*sign(sum1)*abs(nk_one*eps+sum1)^{-spl}%*%sll
                L[which(abs(L)<abs(alpha*Psi*abs(nk_one*eps+sum1)^{-spl})%*%sll)] <- 0

                Psi <- epsn+abs(XX - diag(tcrossprod(L,sum1)/n))
                if (i %% 20==0) { print(i)}
	}

        ipsi <- 1/Psi
        LPsi<-diag(ipsi)%*%L
        LPsiL<-crossprod(L,LPsi)
        for (j in 1:l){
            tmp <- solve(LPsiL+diag(lapla[j,]))
            x_j <- as.vector(X[,j])
            e_sx_n <- as.vector(tcrossprod(tmp,LPsi)%*%x_j)
            E_SX_n[,j] <- e_sx_n
            E_SSXX_n[[j]] <-  tmp + tcrossprod(e_sx_n,e_sx_n)
        }

         Lz=L%*%E_SX_n


	return(list(LZ=Lz,L=L,Z=E_SX_n,Psi=Psi,lapla=lapla))
}



#########################
##Data
make_fabi_data <- function(n,l,p,f1,f2,of1,of2,sd_noise,sd_z_noise,mean_z,sd_z,sd_l_noise,mean_l,sd_l){

    if(!is.numeric(n)) {
        stop("n must be numeric")
    }
    if(!is.numeric(l)) {
        stop("l must be numeric")
    }

    if(!is.numeric(p)) {
        stop("p must be numeric")
    }
    if(!is.numeric(f1)) {
        stop("f1 must be numeric")
    }



    za <- floor(runif(p)*(l/f1)+of1)
    la <- floor(runif(p)*(n/f2)+of2)
    ZC <- list()
    LC <- list()

    X <- matrix(rnorm(l*n,mean = 0, sd = sd_noise),n,l)
    Y <- matrix(0,n,l)
    for (i in 1:p){

        zj <- rnorm(l,mean = 0, sd = sd_z_noise)
        z0 <- rep(0,l)
        zf <- floor(runif(za[i])*l)
        ZC[[i]] <- zf
        z1 <- rnorm(za[i],mean = mean_z, sd = sd_z)
        zj[zf] <- z1
        z0[zf] <- z1

        lj <- rnorm(n,mean = 0, sd = sd_l_noise)
        l0 <- rep(0,n)
        lf <- floor(runif(la[i])*n)
        LC[[i]] <- lf
        l1 <- rnorm(la[i],mean = mean_l, sd = sd_l)
        lsign <- 2*round(runif(la[i]))-1
        l1 <- l1*lsign
        lj[lf] <- l1
        l0[lf] <- l1

        X <- X + lj%*%t(zj)
        Y <- Y + l0%*%t(z0)


    }
    return(list(X=X,Y=Y,ZC=ZC,LC=LC))
}

#########################
##Blocks
make_fabi_data_blocks <- function(n,l,p,f1,f2,of1,of2,sd_noise,sd_z_noise,mean_z,sd_z,sd_l_noise,mean_l,sd_l){

    if(!is.numeric(n)) {
        stop("n must be numeric")
    }
    if(!is.numeric(l)) {
        stop("l must be numeric")
    }

    if(!is.numeric(p)) {
         stop("p must be numeric")
    }
    if(!is.numeric(f1)) {
        stop("f1 must be numeric")
    }


    za <- floor(runif(p)*(l/f1)+of1)
    la <- floor(runif(p)*(n/f2)+of2)
    ZC <- list()
    LC <- list()


    X <- matrix(rnorm(l*n,mean = 0, sd = sd_noise),n,l)
    Y <- matrix(0,n,l)

    for (i in 1:p){


        zj <- rnorm(l,mean = 0, sd = sd_z_noise)
        z0 <- as.vector(rep(0,l))

        tf <- floor(runif(1)*(l-za[i]))
        zf <- tf:(tf+za[i]-1)
        ZC[[i]] <- zf
        z1 <- rnorm(za[i],mean = mean_z, sd = sd_z)
        zj[zf] <- z1
        z0[zf] <- z1

        lj <- rnorm(n,mean = 0, sd = sd_l_noise)
        l0 <- as.vector(rep(0,n))
        uf <- floor(runif(1)*(n-la[i]))
        lf <- uf:(uf+la[i]-1)
        LC[[i]] <- lf
        l1 <- rnorm(la[i],mean = mean_l, sd = sd_l)
        lsign <- 2*round(runif(la[i]))-1
        l1 <- l1*lsign
        lj[lf] <- l1
        l0[lf] <- l1

        X <- X + lj%*%t(zj)
        Y <- Y + l0%*%t(z0)


    }
    return(list(X=X,Y=Y,ZC=ZC,LC=LC))
}

#########################
##Data
make_fabi_data_pos <- function(n,l,p,f1,f2,of1,of2,sd_noise,sd_z_noise,mean_z,sd_z,sd_l_noise,mean_l,sd_l){

    if(!is.numeric(n)) {
        stop("n must be numeric")
    }
    if(!is.numeric(l)) {
        stop("l must be numeric")
    }

    if(!is.numeric(p)) {
        stop("p must be numeric")
    }
    if(!is.numeric(f1)) {
        stop("f1 must be numeric")
    }



    za <- floor(runif(p)*(l/f1)+of1)
    la <- floor(runif(p)*(n/f2)+of2)
    ZC <- list()
    LC <- list()

    X <- matrix(rnorm(l*n,mean = 0, sd = sd_noise),n,l)
    Y <- matrix(0,n,l)
    for (i in 1:p){

        zj <- rnorm(l,mean = 0, sd = sd_z_noise)
        z0 <- rep(0,l)
        zf <- floor(runif(za[i])*l)
        ZC[[i]] <- zf
        z1 <- rnorm(za[i],mean = mean_z, sd = sd_z)
        zj[zf] <- z1
        z0[zf] <- z1

        lj <- rnorm(n,mean = 0, sd = sd_l_noise)
        l0 <- rep(0,n)
        lf <- floor(runif(la[i])*n)
        LC[[i]] <- lf
        l1 <- rnorm(la[i],mean = mean_l, sd = sd_l)
        lj[lf] <- l1
        l0[lf] <- l1

        X <- X + lj%*%t(zj)
        Y <- Y + l0%*%t(z0)


    }
    return(list(X=X,Y=Y,ZC=ZC,LC=LC))
}

#########################
##Blocks
make_fabi_data_blocks_pos<- function(n,l,p,f1,f2,of1,of2,sd_noise,sd_z_noise,mean_z,sd_z,sd_l_noise,mean_l,sd_l){

    if(!is.numeric(n)) {
        stop("n must be numeric")
    }
    if(!is.numeric(l)) {
        stop("l must be numeric")
    }

    if(!is.numeric(p)) {
         stop("p must be numeric")
    }
    if(!is.numeric(f1)) {
        stop("f1 must be numeric")
    }


    za <- floor(runif(p)*(l/f1)+of1)
    la <- floor(runif(p)*(n/f2)+of2)
    ZC <- list()
    LC <- list()


    X <- matrix(rnorm(l*n,mean = 0, sd = sd_noise),n,l)
    Y <- matrix(0,n,l)

    for (i in 1:p){


        zj <- rnorm(l,mean = 0, sd = sd_z_noise)
        z0 <- as.vector(rep(0,l))

        tf <- floor(runif(1)*(l-za[i]))
        zf <- tf:(tf+za[i]-1)
        ZC[[i]] <- zf
        z1 <- rnorm(za[i],mean = mean_z, sd = sd_z)
        zj[zf] <- z1
        z0[zf] <- z1

        lj <- rnorm(n,mean = 0, sd = sd_l_noise)
        l0 <- as.vector(rep(0,n))
        uf <- floor(runif(1)*(n-la[i]))
        lf <- uf:(uf+la[i]-1)
        LC[[i]] <- lf
        l1 <- rnorm(la[i],mean = mean_l, sd = sd_l)
        lj[lf] <- l1
        l0[lf] <- l1

        X <- X + lj%*%t(zj)
        Y <- Y + l0%*%t(z0)


    }
    return(list(X=X,Y=Y,ZC=ZC,LC=LC))
}


# mfsc - matrix factorization with sparseness constraints
#
# Written by Sepp Hochreiter according to
# Patrik O. Hoyer
# 'Non-negative matrix factorization with sparseness constraints'
# Journal of Machine Learning Research  5:1457-1469, 2004.
#
#
#
# SYNTAX:
# res <- mfsc( X, p, sL, sZ, cyc=100)
# res$L
# res$Z
#
# INPUTS:
# X          - data matrix
# p       - number of components (inner dimension of factorization)
# sL         - sparseness of L, in [0,1]
# sZ         - sparseness of Z, in [0,1]
# cyc    - maximal number of iterations
#
# Note: Sparseness is measured on the scale [0,1] where 0 means
# completely distributed and 1 means ultimate sparseness.
#
# NOTE: There is NO CONVERGENCE CRITERION.
#

mfsc <- function( X, p, sL, sZ, cyc=100,norm=1,center=2) {

        X <- as.matrix(X)

        l=ncol(X)
        n=nrow(X)

        if (center>0) {
            if (center<2) {
                X <- X- apply(X, 1, mean)
            } else {
                if (center<3) {
                    X <- X- apply(X, 1, median)
                } else {
                    mo <- estimateMode(X)
                    X <- mo$xu
                }
            }
        }


        if (norm>0) {
            XX <- diag((1/l)*tcrossprod(X)+0.001*as.vector(rep(1,n)))
            dXX <- 1/sqrt(XX)
            X <- dXX*X
        }

# Create initial matrices
L <- matrix (rnorm(n*p),nrow=n,ncol=p)
Z <- matrix (rnorm(l*p),nrow=p,ncol=l)



# Make initial matrices have correct sparseness
L1a <- sqrt(n)-(sqrt(n)-1)*sL
L1s <- sqrt(l)-(sqrt(l)-1)*sZ

for (i in 1:p)
  {
    L[,i] <- projfunc(s=as.vector(L[,i]),k1=L1a,k2=1.0)
    Z[i,] <- projfunc(s=as.vector(Z[i,]),k1=L1s,k2=1.0)
  }


# Calculate initial objective
objhistory <- sum((X-L%*%Z)^2)



# Initial stepsizes
stepsizeL <- 1.0
stepsizeZ <- 1.0



# Start iteration
for (i in 1:cyc)
  {


    # Save old values
    Lold <- L
    Zold <- Z

    # ----- Update Z ---------------------------------------


	# Gradient for Z
	dZ <- t(L)%*%(L%*%Z-X)

        begobj <- objhistory

        newobj <- 2.0*begobj




     # Make sure we decrease the objective!
	while (newobj>begobj) {


         # Take step in direction of negative gradient, and project
          Znew <- Z - stepsizeZ*dZ
	# Make initial matrices have correct sparseness
          for (j in 1:p)
            {
              Znew[j,] <- projfunc(s=as.vector(Znew[j,]),k1=L1s,k2=1.0)
            }


         # Calculate new objective
	    newobj <- sum((X-L%*%Znew)^2)

	    # If the objective decreased, we can continue...
            stepsizeZ <- stepsizeZ/2.0
            if (stepsizeZ<1e-10) {
              return(list(L=L,Z=Z))
            }
        }

	# Slightly increase the stepsize
	stepsizeZ <- stepsizeZ*1.2
	Z <- Znew



    # ----- Update L ---------------------------------------


	# Gradient for L
	dL <- (L%*%Z-X)%*%t(Z)
	begobj <- sum((X-L%*%Z)^2)

        newobj <- 2*begobj

	# Make sure we decrease the objective!
	while (newobj>begobj) {



	    # Take step in direction of negative gradient, and project
	    Lnew <- L - stepsizeL*dL
	    norms <- sqrt(sum(Lnew^2))
            for (j in 1:p)
              {
                Lnew[,j] <- projfunc(s=as.vector(Lnew[,j]),k1=L1a,k2=1.0)
              }

	    # Calculate new objective
	    newobj <- sum((X-Lnew%*%Z)^2)

	    stepsizeL <- stepsizeL/2

            if (stepsizeZ<1e-10) {
              return(list(L=L,Z=Z))
            }


          }

	# Slightly increase the stepsize
	stepsizeL <- stepsizeL*1.2
	L <- Lnew


    # Calculate objective
    newobj <- 0.5*sum(sum((X-L%*%Z)^2))
    objhistory <- newobj



  }
}




# Solves the following problem:
# Given a vector s, find the vector v having sum(abs(v))=k1
# and sum(v^2)=k2 which is closest to s in the euclidian sense.


projfunc <- function(s, k1, k2) {


  N <- length(s)

  isneg <- as.vector(rep(0,N))
  isneg[which(s<0)] <- 1
  ones <- as.vector(rep(1,N))

  s <- abs(s)


# Start by projecting the point to the sum constraint hyperplane
  v <- s + (k1-sum(s))/N

# Initialize zerocoeff (initially, no elements are assumed zero)
  zerocoeff <- which(v<=0)


  j <- 0
  ende <- 0
  while (ende<1) {

    # This does the proposed projection operator
    midpoint <- ones*k1/(N-length(zerocoeff))
    midpoint[zerocoeff] <- 0
    w <- v-midpoint
    a <- sum(w^2)
    b <- 2*crossprod(w,v)
    c <- sum(v^2)-k2
    t <- b^2-4*a*c
    if (t<0) {t <- 0}
    if (a<1e-10) {a <- 1e-10}

    alphap <- (-b+sqrt(t))/(2*a)
    v <- alphap*w + v

    if ((length(which(v<0))==0)&&(j>1)) {
      ende <- 2
    }
    else {

      j <- j+1

    # Set negs to zero, subtract appropriate amount from rest
      zerocoeff <- which(v<=0)
      v[zerocoeff] <- 0
      tempsum <- sum(v)
      tta <- N - length(zerocoeff)
      if (tta>0)  {
       v <- v + (k1-tempsum)/tta
       }
      v[zerocoeff] <- 0
    }
  }

  v <- (-2*isneg + ones)*v

  return(v)

}


fabiasp <- function(X,cyc,alpha,spz,p,norm=1,center=2){
        if(!is.matrix(X)) {
            stop("X must be a matrix");
         }


        eps <- as.double(1e-3)
        eps1 <- as.double(1e-10)
        init_lapla <- 1.0
        init_psi <- 0.2

        X <- as.matrix(X)

        l=ncol(X)
        n=nrow(X)

        if (center>0) {
            if (center<2) {
                X <- X- apply(X, 1, mean)
            } else {
                if (center<3) {
                    X <- X- apply(X, 1, median)
                } else {
                    mo <- estimateMode(X)
                    X <- mo$xu
                }
            }
        }

        XX <- diag((1/l)*tcrossprod(X)+0.001*as.vector(rep(1,n)))

        if (norm>0) {
            dXX <- 1/sqrt(XX)
            X <- dXX*X
            XX <- as.vector(rep(1.0,n))
        }




        lapla <-  init_lapla*matrix(1,nrow=l,ncol=p)

        Psi <- init_psi*XX


        eps2 <- 1e-1



        L1a <- sqrt(n)-(sqrt(n)-1)*alpha

        #y_v <- colMeans(X)
        #xmean <- as.vector(rep(1,l))%*%t(y_v)
        #x <- X-xmean  ## center data (0 mean)
	#XX <- diag(crossprod(x,x)/n)
        #dXX <- diag(diag(XX))
        # idXX <- diag(1/diag(XX))
        #XX <- XX%*%idXX
        #x <- x%*%sqrt(idXX)


        kvect <- as.vector(rep(1,p))
        nvect <- as.vector(rep(1,n))
        nk_one <- matrix(1,n,p)
        nk_zero <- matrix(0,n,p)
        kk_zero <- matrix(0,p,p)
        kk_one <- diag(p)
	epsv<-eps1*kvect
	epsn<-eps*nvect
        E_SX_n <- matrix(0,p,l)
        E_SSXX_n <- list()

        L <- (0.5*XX^(.5))*nk_one
        L <- L/sqrt(sum(L*L))

	for (i in 1:cyc){
                ipsi <- 1/Psi
		LPsi<-diag(ipsi)%*%L
		LPsiL<-crossprod(L,LPsi)
 		sum1<- nk_zero
		sum2<- eps2*kk_one
 		for (j in 1:l){
			tmp <- solve(LPsiL+diag(lapla[j,]))
			x_j <- as.vector(X[,j])
			e_sx_n <- as.vector(tcrossprod(tmp,LPsi)%*%x_j)
			#e_sx_n[which(e_sx_n<0)] <- 0
			e_ssxx_n <-  tmp + tcrossprod(e_sx_n,e_sx_n)
			sum1 <- sum1 +  tcrossprod(x_j,e_sx_n)
			sum2 <- sum2 + e_ssxx_n
			lapla[j,] <- (epsv+diag(e_ssxx_n))^(-spz)
                        lapla[j,which( lapla[j,]<1.0)] <- 1.0
		}
                L <- sum1%*%solve(sum2)

                Psi <- epsn+abs(XX - diag(tcrossprod(L,sum1)/n))

                for (j in 1:p)
                {
                    L[,j] <- projfunc(s=as.vector(L[,j]),k1=L1a,k2=1.0)
                }

                if (i %% 20==0) { print(i)}
	}

        ipsi <- 1/Psi
        LPsi<-diag(ipsi)%*%L
        LPsiL<-crossprod(L,LPsi)
        for (j in 1:l){
            tmp <- solve(LPsiL+diag(lapla[j,]))
            x_j <- as.vector(X[,j])
            e_sx_n <- as.vector(tcrossprod(tmp,LPsi)%*%x_j)
            E_SX_n[,j] <- e_sx_n
            E_SSXX_n[[j]] <-  tmp + tcrossprod(e_sx_n,e_sx_n)
        }

         Lz=L%*%E_SX_n


	return(list(LZ=Lz,L=L,Z=E_SX_n,Psi=Psi,lapla=lapla))
}

extract_plot <- function(X,L,Z,thresZ=0.5,ti,thresL=NULL,Y=NULL,x11b=TRUE,norm=1,center=2){


    X <- as.matrix(X)

    l=ncol(X)
    n=nrow(X)

    if (center>0) {
        if (center<2) {
            X <- X- apply(X, 1, mean)
        } else {
            if (center<3) {
                X <- X- apply(X, 1, median)
            } else {
                mo <- estimateMode(X)
                X <- mo$xu
            }
        }
    }


    XX <- diag((1/l)*tcrossprod(X)+0.001*as.vector(rep(1,n)))

    if (norm>0) {
        dXX <- 1/sqrt(XX)
        X <- dXX*X
        XX <- as.vector(rep(1.0,n))
    }





    p <- ncol(L)


    lll <- 12

    yl <- c(1,round(n*(1:lll)/lll))

    yll <- rep("",n)

    yll[yl] <- as.character(yl)

    LZ <- L%*%Z

    tt <- paste("(",as.character(n)," genes, ",as.character(l)," samples, ",as.character(p)," biclusters )")

    if (!is.null(Y))
    {
      if (x11b) { x11() }
      myImagePlot(Y,title=paste(ti,": noise free data\n",tt,sep=""),yLabels= yll)

    }
    if (x11b) { x11() }
    myImagePlot(X,title=paste(ti,": data\n",tt,sep=""),yLabels= yll)

    if (x11b) { x11() }
    myImagePlot(LZ,title=paste(ti,": reconstructed data\n",tt,sep=""),yLabels= yll)
    if (x11b) { x11() }
    myImagePlot(LZ-X,title=paste(ti,": error\n",tt,sep=""),yLabels= yll)
    if (x11b) { x11() }
    myImagePlot(abs(Z),title=paste(ti,": absolute factors\n",tt,sep=""))
    if (x11b) { x11() }
    myImagePlot(abs(L),title=paste(ti,": absolute loadings\n",tt,sep=""),yLabels= yll)

    iin <-  1.0/l

    vz <- iin*diag(tcrossprod(Z))

    vz <- sqrt(vz)

    ivz <- 1/vz

    nZ <- diag(ivz)%*%Z

    nL <- L%*%diag(vz)

    if (x11b) { x11() }
    myImagePlot(abs(nL),title=paste(ti,": absolute loadings normalized\n",tt,sep=""),yLabels= yll)
    if (x11b) { x11() }
    myImagePlot(abs(nZ),title=paste(ti,": absolute factors normalized\n",tt,sep=""))


    numn1 <- list()
    numn2 <- list()
    numn3 <- list()
    bin <- list()
    bixv <- list()
    bixn <- list()
    biyp <- list()
    biypv <- list()
    biypn <- list()
    biyn <- list()
    biynv <- list()
    biynn <- list()
    gene_names <- rownames(X)
    if (is.null(gene_names))
    {
       gene_names <- as.character(1:n)
    }
    sample_names <- colnames(X)
    if (is.null(sample_names))
    {
       sample_names <- as.character(1:l)
    }

    if (is.null(thresL)) {
        mom <- 0
        for (i in 1:p) {
            tmom <- as.numeric(tcrossprod(nL[,i],nZ[i,]))
            mom <-  mom + sum(tmom^2)
        }
        mom <- mom/(length(tmom)*p)

        thresL <- sqrt(mom)/thresZ
    }

    for (i in 1:p){

        snl <- sort(abs(nL[,i]),decreasing = TRUE,index.return = TRUE)
        gene_namest <- gene_names[snl$ix]
        sL <- nL[snl$ix,i]


        numn1[[i]] <- which(abs(nL[,i])>thresL)
        cho <- which(abs(sL)>thresL)
        bixv[[i]] <- sL[cho]
        bixn[[i]] <- gene_namest[cho]


        snz <- sort(abs(nZ[i,]),decreasing = TRUE,index.return = TRUE)
        sample_namest <- sample_names[snz$ix]
        sz <- nZ[i,snz$ix]

        numn2[[i]] <- which(nZ[i,]>thresZ)
        chop <- which(sz>thresZ)
        biypv[[i]] <- sz[chop]
        biypn[[i]] <-  sample_namest[chop]

        numn3[[i]] <- which(nZ[i,]< -thresZ)
        chon <- which(sz< -thresZ)
        biynv[[i]] <- sz[chon]
        biynn[[i]] <-  sample_namest[chon]

        bin[[i]] <- c(length(cho),length(chop),length(chon))

    }
    bic <- cbind(bin,bixv,bixn,biypv,biypn,biynv,biynn)
    numn <- cbind(numn1,numn2,numn3)




    hnzc <- kmeans(t(nZ),p)

    hnzs <- sort(hnzc$cluster,index.return = TRUE)


    pmZ <- matrix(0,l,l)
    for (i in 1:l){
         pmZ[hnzs$ix[i],i] <- 1
    }

    if (x11b) { x11() }
    myImagePlot(nZ%*%pmZ,title=paste(ti,": factors sorted\n",tt,sep=""))




    hnLc <- kmeans(nL,p)

    hnLs <- sort(hnLc$cluster,index.return = TRUE)

    biclustx <- list()
    biclusty <- list()
    for (i in 1:p){

        biclustx[[i]] <- hnLs$ix[which(hnLs$x==i)]
        biclusty[[i]] <- hnzs$ix[which(hnzs$x==i)]

    }

    biclust <- cbind(biclustx,biclusty)

    pmL <- matrix(0,n,n)
    for (i in 1:n){
         pmL[i,hnLs$ix[i]] <- 1
    }

    if (x11b) { x11() }
    myImagePlot(pmL%*%nL,title=paste(ti,": loadings sorted\n",tt,sep=""), yLabels= yll)

    if (x11b) { x11() }
    myImagePlot(pmL%*%L%*%Z%*%pmZ,title=paste(ti,": reconstructed matrix sorted\n",tt,sep=""), yLabels= yll)

    reord <- pmL%*%as.matrix(X)%*%pmZ

    if (x11b) { x11() }
    myImagePlot(reord,title=paste(ti,": original matrix sorted\n",tt,sep=""), yLabels= yll)


    return(list(bic=bic,numn=numn,biclust=biclust,pmZ=pmZ,pmL=pmL,nL=nL,nZ=nZ,Xord=reord))

}

extract_bic <- function(L,Z,thresZ=0.5,thresL=NULL,lapla=NULL,Psi=NULL){


    n <- nrow(L)

    p <- ncol(L)

    l <- ncol(Z)



    iin <-  1.0/l

    vz <- iin*diag(tcrossprod(Z))

    vz <- sqrt(vz)

    ivz <- 1/vz

    nZ <- diag(ivz)%*%Z

    nL <- L%*%diag(vz)




    binn <- list()
    binp <- list()
    bixv <- list()
    bixn <- list()
    numng <- list()

    numnp <- list()
    biyp <- list()
    biypv <- list()
    biypn <- list()

    numnn <- list()
    biyn <- list()
    biynv <- list()
    biynn <- list()
    gene_names <- rownames(L)
    if (is.null(gene_names))
    {
       gene_names <- as.character(1:n)
    }
    sample_names <- colnames(Z)
    if (is.null(sample_names))
    {
       sample_names <- as.character(1:l)
    }


    if (is.null(thresL)) {
        mom <- 0
        for (i in 1:p) {
            tmom <- as.numeric(tcrossprod(nL[,i],nZ[i,]))
            mom <- mom + sum(tmom^2)
        }
        mom <- mom/(length(tmom)*p)

        thresL <- sqrt(mom)/thresZ
    }


    for (i in 1:p){

        snl <- sort(abs(nL[,i]),decreasing = TRUE,index.return = TRUE)
        gene_namest <- gene_names[snl$ix]
        sL <- nL[snl$ix,i]


        numng[[i]] <- as.vector(which(abs(nL[,i])>thresL))
        cho <- which(abs(sL)>thresL)
        bixv[[i]] <- sL[cho]
        bixn[[i]] <- gene_namest[cho]


        snz <- sort(abs(nZ[i,]),decreasing = TRUE,index.return = TRUE)
        sample_namest <- sample_names[snz$ix]
        sz <- nZ[i,snz$ix]

        chop <- which(sz>thresZ)
        ss2 <- sum(abs(sz[chop]))
        chon <- which(sz< -thresZ)
        ss3 <- sum(abs(sz[chon]))

        if (ss2>=ss3) {
            binp[[i]] <- c(length(cho),length(chop))
            numnp[[i]] <-  as.vector(which(nZ[i,]>thresZ))
            biypv[[i]] <- sz[chop]
            biypn[[i]] <-  sample_namest[chop]

            binn[[i]] <- c(length(cho),length(chon))
            numnn[[i]] <-  as.vector(which(nZ[i,]< -thresZ))
            biynv[[i]] <- sz[chon]
            biynn[[i]] <-  sample_namest[chon]
        } else {
            binn[[i]]    <- c(length(cho),length(chop))
            numnn[[i]]   <-  as.vector(which(nZ[i,]>thresZ))
            biynv[[i]]   <- sz[chop]
            biynn[[i]]   <-  sample_namest[chop]

            binp[[i]]    <- c(length(cho),length(chon))
            numnp[[i]]   <-  as.vector(which(nZ[i,]< -thresZ))
            biypv[[i]]   <- sz[chon]
            biypn[[i]]   <-  sample_namest[chon]

        }





    }

    bic <- cbind(binp,bixv,bixn,biypv,biypn)
    numn <- cbind(numng,numnp)

    bicopp <- cbind(binn,bixv,bixn,biynv,biynn)
    numnopp <- cbind(numng,numnn)

    # INI call for biclusters
    ini <- matrix(0,l,(p+1))
    avini <- as.vector(rep(0.0,(p+1)))

    idp <- diag(p)
    ipsi <- 1/Psi
    if ((!is.null(lapla))&&(!is.null(Psi)))
    {
         for (j in 1:l){
     #   mat <- solve(nL%*%diag(1/lapla[j,])%*%t(nL)+diag(Psi))
         ppL <- crossprod(nL,ipsi*nL)
         mat <- idp + ppL/lapla[j,]
         ini[j,p+1] <- log(det(mat))

         for (i in 1:p){
              ini[j,i] <- log(1+(1/lapla[j,i])*crossprod(nL[,i],ipsi*nL[,i]))
          }
         }
         for (i in 1:p){
             avini[i] <- sum(ini[,i])
         }

         avini[p+1] <- sum(ini[,p+1])
    }



    return(list(bic=bic,numn=numn,bicopp=bicopp,numnopp=numnopp,avini=avini,ini=ini))

}





nmfdiv <- function(X, p, cyc =100) {

# Check that we have non-negative data
if (min(X)<0) {
  print("Negative values in data!")
  return(list(NULL,NULL))
}

# Globally rescale data to avoid potential overflow/underflow
X <- X/max(X)

# Dimensions
n <- nrow(X)
l <- ncol(X)


# Create initial matrices
L <- matrix (abs(rnorm(n*p)),nrow=n,ncol=p)
Z <- matrix (abs(rnorm(l*p)),nrow=p,ncol=l)
#Z <- Z/(sqrt(rowsum(Z*Z,1:n))*as.vector(rep(1,l)))

# Calculate initial objective
objhistory <- 0.5*sum(sum((X-L%*%Z)^2))




# Start iteration
for (i in 1:cyc)
  {

    # Show progress
    print(i)
    print(objhistory)

    # Save old values
    Lold <- L
    Zold <- Z


    # Compute new L and Z (Lee and Seung; NIPS*2000)

    #L <- L*((X/(L%*%Z + 1e-9))%*%t(Z))/rowSums(Z)
    #Z <- Z*(t(L)%*%(X/(L%*%Z + 1e-9)))/colSums(L)

    L <- L*((X/(L%*%Z + 1e-9))%*%t(Z))/tcrossprod(rep(1,n),rowSums(Z))
    Z <- Z*(t(L)%*%(X/(L%*%Z + 1e-9)))/tcrossprod(colSums(L),rep(1,l))



    # Renormalize so rows of Z have constant energy
    norms <- sqrt(rowSums(Z^2))
#    Z <- Z/tcrossprod(norms,rep(1,l))
#    L <- L*tcrossprod(rep(1,n),norms)

    Z <- Z/norms
    L <- L*norms

    # Calculate objective
    newobj <- sum(X*log((X+1e-10)/(L%*%Z)) - X + L%*%Z)
    objhistory <- newobj


  }
   return(list(L=L,Z=Z))
}


nmfeu <- function(X, p, cyc=100) {

# Check that we have non-negative data
if (min(X)<0) {
  print("Negative values in data!")
  return(list(NULL,NULL))
}

# Globally rescale data to avoid potential overflow/underflow
X <- X/max(X)

# Dimensions
n <- nrow(X)
l <- ncol(X)


# Create initial matrices
L <- matrix (abs(rnorm(n*p)),nrow=n,ncol=p)
Z <- matrix (abs(rnorm(l*p)),nrow=p,ncol=l)
#Z <- Z/(sqrt(rowsum(Z*Z,1:n))*as.vector(rep(1,l)))

# Calculate initial objective
objhistory <- 0.5*sum(sum((X-L%*%Z)^2))


# Start iteration
for (i in 1:cyc)
  {

    # Show progress
    print(i)
    print(objhistory)

    # Save old values
    Lold <- L
    Zold <- Z


    # Compute new L and Z (Lee and Seung; NIPS*2000)
    Z <- Z*(t(L)%*%X)/(t(L)%*%L%*%Z + 1e-9)
    L <- L*(X%*%t(Z))/(L%*%Z%*%t(Z) + 1e-9)

    # Renormalize so rows of Z have constant energy
    norms <- sqrt(rowSums(Z^2))
#    Z <- Z/tcrossprod(norms,rep(1,l))
#    L <- L*tcrossprod(rep(1,n),norms)

    Z <- Z/norms
    L <- L*norms



    # Calculate objective
    newobj <- sum(X*log((X+1e-10)/(L%*%Z)) - X + L%*%Z)
    objhistory <- newobj


  }
   return(list(L=L,Z=Z))
}


nprojfunc <- function(s, k1, k2) {


  N <- length(s)


  ones <- as.vector(rep(1,N))



# Start by projecting the point to the sum constraint hyperplane
  v <- s + (k1-sum(s))/N

# Initialize zerocoeff (initially, no elements are assumed zero)
  zerocoeff <- which(v<=0)


  j <- 0
  ende <- 0
  while (ende<1) {

    # This does the proposed projection operator
    midpoint <- ones*k1/(N-length(zerocoeff))
    midpoint[zerocoeff] <- 0
    w <- v-midpoint
    a <- sum(w^2)
    b <- 2*crossprod(w,v)
    c <- sum(v^2)-k2
    t <- b^2-4*a*c
    if (t<0) {t <- 0}
    if (a<1e-10) {a <- 1e-10}

    alphap <- (-b+sqrt(t))/(2*a)
    v <- alphap*w + v

    if ((length(which(v<0))==0)&&(j>1)) {
      ende <- 2
    }
    else {

      j <- j+1

    # Set negs to zero, subtract appropriate amount from rest
      zerocoeff <- which(v<=0)
      v[zerocoeff] <- 0
      tempsum <- sum(v)
      tta <- N - length(zerocoeff)
      if (tta>0)  {
       v <- v + (k1-tempsum)/tta
       }
      v[zerocoeff] <- 0
    }
  }


  return(v)

}




# nmfsc - nonnegative matrix factorization with sparseness constraints
#
# Written by Sepp Hochreiter according to
# Patrik O. Hoyer
# 'Non-negative matrix factorization with sparseness constraints'
# Journal of Machine Learning Research  5:1457-1469, 2004.
#
#
# SYNTAX:
# res <- nmfsc( X, p, sL, sZ,cyc=100)
# res$L
# res$Z
#
# INPUTS:
# X          - data matrix
# p       - number of components (inner dimension of factorization)
# sL         - sparseness of L, in [0,1]
# sZ         - sparseness of Z, in [0,1]
# cyc    - maximal number of iterations
#
# Note: Sparseness is measured on the scale [0,1] where 0 means
# completely distributed and 1 means ultimate sparseness.
#
# NOTE: There is NO CONVERGENCE CRITERION.
#

nmfsc <- function( X, p, sL, sZ,cyc=100) {

# Data dimensions
n <- nrow(X)
l <- ncol(X)

# Create initial matrices
L <- matrix (rnorm(n*p),nrow=n,ncol=p)
Z <- matrix (rnorm(l*p),nrow=p,ncol=l)



# Make initial matrices have correct sparseness
L1a <- sqrt(n)-(sqrt(n)-1)*sL
L1s <- sqrt(l)-(sqrt(l)-1)*sZ

for (i in 1:p)
  {
    L[,i] <- nprojfunc(s=as.vector(L[,i]),k1=L1a,k2=1.0)
    Z[i,] <- nprojfunc(s=as.vector(Z[i,]),k1=L1s,k2=1.0)
  }


# Calculate initial objective
objhistory <- sum((X-L%*%Z)^2)



# Initial stepsizes
stepsizeL <- 1.0
stepsizeZ <- 1.0



# Start iteration
for (i in 1:cyc)
  {


    # Save old values
    Lold <- L
    Zold <- Z

    # ----- Update Z ---------------------------------------


	# Gradient for Z
	dZ <- t(L)%*%(L%*%Z-X)

        begobj <- objhistory

        newobj <- 2.0*begobj




     # Make sure we decrease the objective!
	while (newobj>begobj) {


         # Take step in direction of negative gradient, and project
          Znew <- Z - stepsizeZ*dZ
	# Make initial matrices have correct sparseness
          for (j in 1:p)
            {
              Znew[j,] <- nprojfunc(s=as.vector(Znew[j,]),k1=L1s,k2=1.0)
            }


         # Calculate new objective
	    newobj <- sum((X-L%*%Znew)^2)

	    # If the objective decreased, we can continue...
            stepsizeZ <- stepsizeZ/2.0
            if (stepsizeZ<1e-10) {
              return(list(L=L,Z=Z))
            }
        }

	# Slightly increase the stepsize
	stepsizeZ <- stepsizeZ*1.2
	Z <- Znew



    # ----- Update L ---------------------------------------


	# Gradient for L
	dL <- (L%*%Z-X)%*%t(Z)
	begobj <- sum((X-L%*%Z)^2)

        newobj <- 2*begobj

	# Make sure we decrease the objective!
	while (newobj>begobj) {



	    # Take step in direction of negative gradient, and project
	    Lnew <- L - stepsizeL*dL
	    norms <- sqrt(sum(Lnew^2))
            for (j in 1:p)
              {
                Lnew[,j] <- nprojfunc(s=as.vector(Lnew[,j]),k1=L1a,k2=1.0)
              }

	    # Calculate new objective
	    newobj <- sum((X-Lnew%*%Z)^2)

	    stepsizeL <- stepsizeL/2

            if (stepsizeZ<1e-10) {
              return(list(L=L,Z=Z))
            }


          }

	# Slightly increase the stepsize
	stepsizeL <- stepsizeL*1.2
	L <- Lnew


    # Calculate objective
    newobj <- 0.5*sum(sum((X-L%*%Z)^2))
    objhistory <- newobj



  }
}

