\name{fabiasp}
\alias{fabiasp}
\title{Factor Analysis for Bicluster Acquisition: Sparseness Projection (FABIASP)}
\description{

  \code{fabiasp}: \R implementation of  \code{fabias}, therefore it is \bold{slow}.
}
\usage{

fabiasp(X,cyc,alpha,spz,p,norm=1,center=2)

}
\arguments{
  \item{X}{the data matrix.}
  \item{cyc}{number of cycles to run.}
  \item{alpha}{sparseness loadings via projection (0.1 - 0.9).}
  \item{spz}{sparseness factors (0.5 - 4.0).}
  \item{p}{number of hidden factors = number of biclusters.}
  \item{norm}{data normalization: default = 1 (yes), 0 (no).}
  \item{center}{data centering: 1 (mean), 2
    (median), > 2 (mode), 0 (no); default 2.}
}
\details{

  Biclusters are found by sparse factor analysis where \emph{both} the factors
  and the loadings are sparse.

  Essentially the model is the sum of
  outer products of vectors:
   \deqn{X \ = \ \sum_{i=1}^{p} \lambda_i \
                   z_i^T \ + \ U}
  where the number of summands  $p$
  is the number of biclusters.
  The matrix factorization is
  \deqn{X \ = \ L \ Z \ + \ U}.

  If the nonzero components of
  the sparse vectors are grouped together then the outer product results in
  a matrix with a nonzero block and zeros elsewhere.


  The model selection is performed by a variational approach according
  to Girolami 2001 and Palmer et al. 2006.

  The prior has finite support, therefore after each
  update of the loadings they are projected to the finite support.
  The projection is done according to Hoyer, 2004: given an
  \eqn{l_1}-norm and an \eqn{l_2}-norm minimize the Euclidian distance
  to the original vector (currently the \eqn{l_2}-norm is fixed to 1).
  The projection is a convex quadratic problem which is solved
  iteratively where at each iteration at least one component is set to
  zero. Instead of the  \eqn{l_1}-norm a sparseness measurement is used
  which relates the \eqn{l_1}-norm to the \eqn{l_2}-norm.

   The code is implemented in \R, therefore it is \bold{slow}.

}
\value{
  \item{LZ}{Estimated Noise Free Data:
    \eqn{L \ Z}}
  \item{L}{Loadings:  \eqn{L}}
  \item{Z}{Factors:  \eqn{Z}}
  \item{Psi}{Noise variance: \eqn{\sigma}}
  \item{lapla}{Variational parameter}
    }
\seealso{
\code{\link{fabi}},
\code{\link{fabia}},
\code{\link{fabiap}},
\code{\link{fabias}},
\code{\link{mfsc}},
\code{\link{nmfdiv}},
\code{\link{nmfeu}},
\code{\link{nmfsc}},
\code{\link{nprojfunc}},
\code{\link{projfunc}},
\code{\link{estimateMode}},
\code{\link{make_fabi_data}},
\code{\link{make_fabi_data_blocks}},
\code{\link{make_fabi_data_pos}},
\code{\link{make_fabi_data_blocks_pos}},
\code{\link{extract_plot}},
\code{\link{extract_bic}},
\code{\link{myImagePlot}},
\code{\link{PlotBicluster}},
\code{\link{Breast_A}},
\code{\link{DLBCL_B}},
\code{\link{Multi_A}},
\code{\link{fabiaDemo}},
\code{\link{fabiaVersion}}
}
\author{Sepp Hochreiter}
\examples{

#---------------
# TEST
#---------------

dat <- make_fabi_data_blocks(n = 100,l= 50,p = 3,f1 = 5,f2 = 5,
  of1 = 5,of2 = 10,sd_noise = 3.0,sd_z_noise = 0.2,mean_z = 2.0,
  sd_z = 1.0,sd_l_noise = 0.2,mean_l = 3.0,sd_l = 1.0)

X <- dat[[1]]
Y <- dat[[2]]



resEx <- fabiasp(X,50,0.8,1.0,3)


\dontrun{
#---------------
# DEMO1
#---------------

dat <- make_fabi_data_blocks(n = 1000,l= 100,p = 10,f1 = 5,f2 = 5,
  of1 = 5,of2 = 10,sd_noise = 3.0,sd_z_noise = 0.2,mean_z = 2.0,
  sd_z = 1.0,sd_l_noise = 0.2,mean_l = 3.0,sd_l = 1.0)

X <- dat[[1]]
Y <- dat[[2]]


resToy <- fabiasp(X,200,0.6,1.0,13)

rToy <- extract_plot(X,resToy$L,resToy$Z,"ti=FABIASP",Y=Y)

#---------------
# DEMO2
#---------------


data(Breast_A)

X <- as.matrix(XBreast)

resBreast <- fabiasp(X,200,0.4,1.0,5)

rBreast <- extract_plot(X,resBreast$L,resBreast$Z,ti="FABIASP Breast cancer(Veer)")

#sorting of predefined labels
CBreast%*%rBreast$pmZ

#---------------
# DEMO3
#---------------


data(Multi_A)

X <- as.matrix(XMulti)

resMulti <- fabiasp(X,200,0.4,1.0,5)

rMulti <- extract_plot(X,resMulti$L,resMulti$Z,"ti=FABIASP Multiple tissues(Su)")

#sorting of predefined labels
CMulti%*%rMulti$pmZ

#---------------
# DEMO4
#---------------


data(DLBCL_B)

X <- as.matrix(XDLBCL)


resDLBCL <- fabiasp(X,200,0.6,1.0,5)

rDLBCL <- extract_plot(X,resDLBCL$L,resDLBCL$Z,ti="FABIASP Lymphoma(Rosenwald)")

#sorting of predefined labels
CDLBCL%*%rDLBCL$pmZ

}
}
\references{
  Mark Girolami,
  \sQuote{A Variational Method for Learning Sparse and
  Overcomplete Representations},
  Neural Computation 13(11): 2517-2532, 2001.

 J. Palmer, D. Wipf, K. Kreutz-Delgado, B. Rao,
  \sQuote{Variational EM algorithms for
     non-Gaussian latent variable models},
     Advances in Neural Information Processing
     Systems 18, pp. 1059-1066, 2006.
     
  Patrik O. Hoyer,
  \sQuote{Non-negative Matrix Factorization with Sparseness Constraints},
  Journal of Machine Learning Research 5:1457-1469, 2004.
}
\keyword{methods}
\keyword{multivariate}
\keyword{cluster}
\concept{biclustering}
\concept{factor analysis}
\concept{sparse coding}
\concept{Laplace distribution}
\concept{EM algorithm}
\concept{nonnegative matrix factorization}
\concept{multivariate analysis}
\concept{latent variables}


