\name{nmfsc}
\alias{nmfsc}
\title{Non-negative Sparse Matrix Factorization}
\description{

  \code{nmfsc}: \R implementation of  \code{nmfsc}.
}
\usage{

nmfsc(X,p,sL,sZ,cyc=100)


}
\arguments{
  \item{X}{the data matrix.}
  \item{p}{number of hidden factor = number of biclusters.}
  \item{sL}{sparseness loadings.}
  \item{sZ}{sparseness factors.}
  \item{cyc}{maximal number of iterations.}
}
\details{


     Non-negative Matrix Factorization represents positive matrix
  \eqn{X} by positive matrices \eqn{L} and \eqn{Z} that are sparse.

  Objective for reconstruction is Euclidean distance and sparseness
  constraints.



  \deqn{X \ = \ L \ Z }

  \deqn{X \ = \ \sum_{i=1}^{p} L_i \
                   \left(Z_i \right)^T}


  If the nonzero components of
  the sparse vectors are grouped together then the outer product results in
  a matrix with a nonzero block and zeros elsewhere.



	 The model selection is performed by a constraint optimization
	 according
  to  Hoyer, 2004. The Euclidean distance (the Frobenius norm) is
  minimized  subject to sparseness and non-negativity constraints.

  Model selection is done by gradient descent on the Euclidean
  objective and thereafter projection of single vectors of
  \eqn{L} and single vectors of \eqn{Z}
  to fulfill the sparseness and non-negativity constraints.

  The projection minimize the Euclidean distance
  to the original vector  given an
  \eqn{l_1}-norm and an \eqn{l_2}-norm and enforcing non-negativity.

  The projection is a convex quadratic problem which is solved
  iteratively where at each iteration at least one component is set to
  zero. Instead of the  \eqn{l_1}-norm a sparseness measurement is used
  which relates the \eqn{l_1}-norm to the \eqn{l_2}-norm.

  The code is implemented in \R.

}
\value{
  \item{L}{Left matrix:  \eqn{L}}
  \item{Z}{Right matrix:  \eqn{Z}}
    }
\seealso{
\code{\link{fabi}},
\code{\link{fabia}},
\code{\link{fabiap}},
\code{\link{fabias}},
\code{\link{fabiasp}},
\code{\link{mfsc}},
\code{\link{nmfdiv}},
\code{\link{nmfeu}},
\code{\link{nprojfunc}},
\code{\link{projfunc}},
\code{\link{make_fabi_data}},
\code{\link{make_fabi_data_blocks}},
\code{\link{make_fabi_data_pos}},
\code{\link{make_fabi_data_blocks_pos}},
\code{\link{extract_plot}},
\code{\link{extract_bic}},
\code{\link{myImagePlot}},
\code{\link{PlotBicluster}},
\code{\link{Breast_A}},
\code{\link{DLBCL_B}},
\code{\link{Multi_A}},
\code{\link{fabiaDemo}},
\code{\link{fabiaVersion}}
}
\author{Sepp Hochreiter}
\examples{

#---------------
# TEST
#---------------

dat <- make_fabi_data_blocks(n = 100,l= 50,p = 3,f1 = 5,f2 = 5,
  of1 = 5,of2 = 10,sd_noise = 3.0,sd_z_noise = 0.2,mean_z = 2.0,
  sd_z = 1.0,sd_l_noise = 0.2,mean_l = 3.0,sd_l = 1.0)

X <- dat[[1]]
Y <- dat[[2]]
X <- abs(X)
XX <- tcrossprod(X)
dXX <- 1/sqrt(diag(XX))
X <- dXX*X


resEx <- nmfsc(as.matrix(abs(X)),3,0.7,0.7)


\dontrun{
#---------------
# DEMO
#---------------

dat <- make_fabi_data_blocks(n = 1000,l= 100,p = 10,f1 = 5,f2 = 5,
  of1 = 5,of2 = 10,sd_noise = 3.0,sd_z_noise = 0.2,mean_z = 2.0,
  sd_z = 1.0,sd_l_noise = 0.2,mean_l = 3.0,sd_l = 1.0)

X <- dat[[1]]
Y <- dat[[2]]
X <- abs(X)
XX <- tcrossprod(X)
dXX <- 1/sqrt(diag(XX))
X <- dXX*X


resToy <- nmfsc(as.matrix(abs(X)),8,0.7,0.7)

rToy <- extract_plot(X,resToy$L,resToy$Z,ti="NMFSC",Y=Y)

}
}
\references{
  Patrik O. Hoyer,
  \sQuote{Non-negative Matrix Factorization with Sparseness Constraints},
  Journal of Machine Learning Research 5:1457-1469, 2004.

  D. D. Lee and H. S. Seung,
  \sQuote{Algorithms for non-negative matrix factorization},
  In Advances in Neural Information Processing Systems 13, 556-562,
  2001.
}
\keyword{methods}
\keyword{cluster}
\concept{sparse coding}
\concept{non-negative matrix factorization}
